/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     ltree_branchH
#define     ltree_branchH

#include    "main.h"

namespace tree{

//-----------------------------------------------------------------------------
// Node of a tree

class TNode{

    protected:

        int id = -1;        // ID of this node (shoul be unique in a tree)
        int parent = -1;    // ID of a parent of this node
        vector<int> childs; // IDs of childs of this node
        string name;        // Name of this node
        string attribute;   // Name of an attribute of this node

    public:

        // Fields access
        int Id(void) const{
            return id;
            }
        int ParentId(void) const{
            return parent;
            }
        const vector<int>& Childs(void) const{
            return childs;
            }
        const string& Name(void) const{
            return name;
            }
        const string& Attribute(void) const{
            return attribute;
            }

        // Modifiers
        void WriteAttribute(const string &attr){
            attribute = attr;
            }
        void AddChild(const int child){
            childs.push_back(child);
            }

        // Comparison of nodes
        bool operator==(const TNode &node) const{
            return (id == node.id && parent == node.parent && name == node.name);
            }
        bool Compare(const TNode &node) const{
            return *this == node;
            }

        void Clear(void){
            id = -1;
            parent = -1;
            childs.clear();
            name.clear();
            attribute.clear();
            }

        TNode(void){}
        TNode(const string &n, const int unique_id, const int parent_id){
            name = n;
            id = unique_id;
            parent = parent_id;
            }
    };

//-----------------------------------------------------------------------------
// Branch of a tree - vector of pointers to nodes of a tree

class TBranch : public vector<TNode>{

    private:

        int branch_mark = -1;   // If > 0 then this node is a last leaf of this branch
        int master_branch = -1; // Branch mark of a master branch (-1 if there is no master branch)
        bool index_enabled = false; // Whether or not an index should be appended to each found value

    public:

        int BranchMark(void) const{
            return branch_mark;
            }
        int MasterBranch(void) const{
            return master_branch;
            }
        bool IndexEnabled(void) const{
            return index_enabled;
            }
        void WriteBranchMark(const int b){
            branch_mark = b;
            }
        void WriteMasterBranch(const int m){
            master_branch = m;
            }
        void WriteIndexFlag(const bool f){
            index_enabled = f;
            }

        #ifdef LTREE_DEBUGGING
        void Print(void) const;
        #endif
    };

//-----------------------------------------------------------------------------
// Branch draft, a vector of this type should be passed to TTree::Construct

class TBranchDraft{

    private:

        vector<string> nodes;
        string attr;
        // Branch mark for this branch
        int mark = -1;
        // Branch mark of a master branch if this branch
        // is an additional branch, -1 otherwise
        int master_branch = -1;
        // Whether or not an index should be appended to each found value
        bool index_enabled = false;

    public:

        // Getters
        const vector<string>& Nodes(void) const{
            return nodes;
            }
        const string& Attr(void) const{
            return attr;
            }
        int BranchMark(void) const{
            return mark;
            }
        int MasterBranch(void) const{
            return master_branch;
            }
        bool IndexEnabled(void) const{
            return index_enabled;
            }

        // Modifiers
        bool Add(const vector<string> &new_nodes, const string &new_attr="");
        bool Complete(const int this_branch_bm, const int master_branch_bm=-1,
            const bool enable_indexing=false);

        // Clear all data of this branch draft
        void Clear(void){
            nodes.clear();
            attr.clear();
            mark = -1;
            master_branch = -1;
            index_enabled = false;
            }

        #ifdef LTREE_DEBUGGING
        void Print(void) const;
        #endif
    };

//-----------------------------------------------------------------------------

} // namespace tree

#endif
